@@ -3,8 +3,11 @@
 // https://github.com/AcademySoftwareFoundation/OpenImageIO
 
 
+#include <OpenImageIO/color.h>
 #include <OpenImageIO/filesystem.h>
+#include <OpenImageIO/fmath.h>
 #include <OpenImageIO/imageio.h>
+#include <OpenImageIO/platform.h>
 #include <OpenImageIO/tiffutils.h>
 
 #include <libheif/heif_cxx.h>
@@ -26,7 +29,11 @@ public:
     const char* format_name(void) const override { return "heif"; }
     int supports(string_view feature) const override
     {
-        return feature == "alpha" || feature == "exif" || feature == "tiles";
+        return feature == "alpha" || feature == "exif" || feature == "tiles"
+#if LIBHEIF_HAVE_VERSION(1, 9, 0)
+               || feature == "cicp"
+#endif
+            ;
     }
     bool open(const std::string& name, const ImageSpec& spec,
               OpenMode mode) override;
@@ -45,6 +52,7 @@ private:
     heif::Encoder m_encoder { heif_compression_HEVC };
     std::vector<unsigned char> scratch;
     std::vector<unsigned char> m_tilebuffer;
+    int m_bitdepth = 0;
 };
 
 
@@ -104,19 +112,33 @@ HeifOutput::open(const std::string& name, const ImageSpec& newspec,
 
     m_filename = name;
 
-    m_spec.set_format(TypeUInt8);  // Only uint8 for now
+    m_bitdepth = m_spec.format.size() > TypeUInt8.size() ? 10 : 8;
+    m_bitdepth = m_spec.get_int_attribute("oiio:BitsPerSample", m_bitdepth);
+    if (m_bitdepth == 10 || m_bitdepth == 12) {
+        m_spec.set_format(TypeUInt16);
+    } else if (m_bitdepth == 8) {
+        m_spec.set_format(TypeUInt8);
+    } else {
+        errorfmt("Unsupported bit depth {}", m_bitdepth);
+        return false;
+    }
 
     try {
         m_ctx.reset(new heif::Context);
         m_himage = heif::Image();
         static heif_chroma chromas[/*nchannels*/]
             = { heif_chroma_undefined, heif_chroma_monochrome,
-                heif_chroma_undefined, heif_chroma_interleaved_RGB,
-                heif_chroma_interleaved_RGBA };
+                heif_chroma_undefined,
+                (m_bitdepth == 8) ? heif_chroma_interleaved_RGB
+                : littleendian()  ? heif_chroma_interleaved_RRGGBB_LE
+                                  : heif_chroma_interleaved_RRGGBB_BE,
+                (m_bitdepth == 8) ? heif_chroma_interleaved_RGBA
+                : littleendian()  ? heif_chroma_interleaved_RRGGBBAA_LE
+                                  : heif_chroma_interleaved_RRGGBBAA_BE };
         m_himage.create(newspec.width, newspec.height, heif_colorspace_RGB,
                         chromas[m_spec.nchannels]);
         m_himage.add_plane(heif_channel_interleaved, newspec.width,
-                           newspec.height, 8 * m_spec.nchannels /*bit depth*/);
+                           newspec.height, m_bitdepth);
 
         m_encoder      = heif::Encoder(heif_compression_HEVC);
         auto compqual  = m_spec.decode_compression_metadata("", 75);
@@ -161,7 +183,22 @@ HeifOutput::write_scanline(int y, int /*z*/, TypeDesc format, const void* data,
     uint8_t* hdata = m_himage.get_plane(heif_channel_interleaved, &hystride);
 #endif
     hdata += hystride * (y - m_spec.y);
-    memcpy(hdata, data, hystride);
+    if (m_bitdepth == 10 || m_bitdepth == 12) {
+        const uint16_t* data16  = static_cast<const uint16_t*>(data);
+        uint16_t* hdata16       = reinterpret_cast<uint16_t*>(hdata);
+        const size_t num_values = m_spec.width * m_spec.nchannels;
+        if (m_bitdepth == 10) {
+            for (size_t i = 0; i < num_values; ++i) {
+                hdata16[i] = bit_range_convert<16, 10>(data16[i]);
+            }
+        } else {
+            for (size_t i = 0; i < num_values; ++i) {
+                hdata16[i] = bit_range_convert<16, 12>(data16[i]);
+            }
+        }
+    } else {
+        memcpy(hdata, data, hystride);
+    }
     return true;
 }
 
@@ -207,8 +244,33 @@ HeifOutput::close()
         } else if (compqual.first == "none") {
             m_encoder.set_lossless(true);
         }
+        heif::Context::EncodingOptions options;
+#if LIBHEIF_HAVE_VERSION(1, 9, 0)
+        // Write CICP. we can only set output_nclx_profile with the C API.
+        std::unique_ptr<heif_color_profile_nclx,
+                        void (*)(heif_color_profile_nclx*)>
+            nclx(heif_nclx_color_profile_alloc(), heif_nclx_color_profile_free);
+        const ColorConfig& colorconfig(ColorConfig::default_colorconfig());
+        const ParamValue* p    = m_spec.find_attribute("CICP",
+                                                       TypeDesc(TypeDesc::INT, 4));
+        string_view colorspace = m_spec.get_string_attribute("oiio:ColorSpace");
+        cspan<int> cicp        = (p) ? p->as_cspan<int>()
+                                     : colorconfig.get_cicp(colorspace);
+        if (!cicp.empty()) {
+            nclx->color_primaries          = heif_color_primaries(cicp[0]);
+            nclx->transfer_characteristics = heif_transfer_characteristics(
+                cicp[1]);
+            nclx->matrix_coefficients   = heif_matrix_coefficients(cicp[2]);
+            nclx->full_range_flag       = cicp[3];
+            options.output_nclx_profile = nclx.get();
+            // Chroma subsampling is incompatible with RGB.
+            if (nclx->matrix_coefficients == heif_matrix_coefficients_RGB_GBR) {
+                m_encoder.set_string_parameter("chroma", "444");
+            }
+        }
+#endif
         encode_exif(m_spec, exifblob, endian::big);
-        m_ihandle = m_ctx->encode_image(m_himage, m_encoder);
+        m_ihandle = m_ctx->encode_image(m_himage, m_encoder, options);
         std::vector<char> head { 'E', 'x', 'i', 'f', 0, 0 };
         exifblob.insert(exifblob.begin(), head.begin(), head.end());
         try {
